/*
 * Graphics
 * (C) 2006 Väinö Helminen
 * $Id: graphics.c,v 1.3 2006-12-09 03:40:58 vah Exp $
 */
#include "graphics.h"

#define BPP 16

inline SDL_Surface* initGraphics(void)
{
  SDL_Surface *screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, BPP, SDL_SWSURFACE);
  if(screen) {
    SDL_ShowCursor(SDL_DISABLE);
  }
  return screen;
}

#if BPP == 8
inline void setPixel(SDL_Surface *surface, int x, int y, Uint32 color)
{
  ((Uint8*)surface->pixels)[y*surface->pitch + x] = color;
}

#else
# if BPP == 16
inline void setPixel(SDL_Surface *surface, int x, int y, Uint32 color)
{
  ((Uint16*)surface->pixels)[y*(surface->pitch>>1) + x] = color;
}
# else
#  error Unsupported bit depth.
# endif
#endif

inline void fillRect(SDL_Surface *surface, int x, int y, int w, int h, Uint32 color)
{
  SDL_Rect r;
  r.x = x; r.y = y; r.w = w; r.h = h;
  SDL_FillRect(surface, &r, color);
}

void scrollOut(SDL_Surface *screen, int x, int y, int w, int h, Uint32 background)
{
  SDL_Rect src, dst;
  int i, ymax = y+h;

  src.x = x; src.y = y; src.w = w, src.h = h-1;
  dst.x = x; dst.y = y+1;
  SDL_BlitSurface(screen, &src, screen, &dst);

  dst.y = y;
  dst.w = w;
  dst.h = 1;
  SDL_FillRect(screen, &dst, background);

  SDL_Flip(screen);

  for(y++, i = 0; y < ymax; y += ++i) {
    dst.y = y;
    SDL_BlitSurface(screen, &src, screen, &dst);
    SDL_Flip(screen);
  }
}
