/*
 * Text drawing
 * (C) 2006 Väinö Helminen
 * $Id: text.c,v 1.1.1.1 2006-12-01 19:26:00 vah Exp $
 */
#include <string.h>

#include "text.h"
#include "font.h"
#include "graphics.h"

#define TAB_SIZE (8*FONT_WIDTH) /* assumed to be a power of two, eg. 8x8=64=2^6 */

static inline void drawChar(int c, int x, int y, SDL_Surface *surface, Uint32 color)
{
  if(c < FONT_OFFSET || c > 255) return;

#if FONT_OFFSET 
  c -= FONT_OFFSET;
#endif
  
  c *= FONT_HEIGHT;
  for(int j = 0; j < FONT_HEIGHT; ++j) {
    int d = FONT_DATA[c+j];
    for(int i = FONT_WIDTH; d ; d >>=1) {
      --i;
      if(d & 1)
         setPixel(surface, x+i, y+j, color);
    } /* for i */
  } /* for j */
}

void alignString(const char *string, Alignment align, int x, int y, SDL_Surface *surface, Uint32 color)
{
  switch(align) {
  default:
  case LEFT:
    break;
  case RIGHT:
    x += surface->w - 1 - FONT_WIDTH * strlen(string);
    break;
  case CENTER:
    x += (surface->w-1 - FONT_WIDTH * strlen(string)) / 2;
  }
  for(; *string; string++, x+=FONT_WIDTH) {
    drawChar(*string, x, y, surface, color);
  }
}

void drawText(const char *string, int x, int y, SDL_Surface *surface, Uint32 color)
{
  int ix = x, iy = y;

  if(!string || !surface) return;

  while(1) {
    int c = *string;
    switch(c) {
    case 0: 
      return;
    case '\r':
      ix = x;
      break;
    case '\n':
      ix = x;
      iy += FONT_HEIGHT;
      break;
    case '\b':
      ix -= FONT_WIDTH;
      break;
    case '\t':
      ix += TAB_SIZE - ((ix-x) & (TAB_SIZE-1)); /* with weird FONT_WIDTH/TAB_SIZE use %-operator instead */
      break;
    case ' ':
      ix += FONT_WIDTH;
      break;
    default:
      drawChar(c, ix, iy, surface, color);
      ix += FONT_WIDTH;
      break;
    } /* switch */
    ++string;
  } /* while 1 */
} 
